//
//  TLDGLineChartController.swift
//  iPhone
//
//  Created by Toj on 5/29/24.
//

import UIKit
import DGCharts

//x轴默认的数据格式也是double类型，所以我们需要像上面取设置y轴的数据格式那样来设置x轴的日期，我们新建DateValueFormatter类遵循协议，然后重写init方法，将x轴的数组传过去，在DateValueFormatter类中定义一个全局数组，在协议方法中去返回数据
class DataValueFormatter: AxisValueFormatter {
    
    func stringForValue(_ value: Double, axis: DGCharts.AxisBase?) -> String {
        return "\(value) x nb"
    }
}

extension DataValueFormatter: ValueFormatter {
    
    func stringForValue(_ value: Double,
                        entry: ChartDataEntry,
                        dataSetIndex: Int,
                        viewPortHandler: ViewPortHandler?) -> String {
        return "\(value) y nb"
    }
}

// MARK: - 懒加载(属性), 变量, 初始化init
class TLDGLineChartController: UIViewController {
    
    /// 折线
    private lazy var lineChartView: LineChartView = {
        let view = setupChartView()
        return view
    }()
    
    /** markerLabel */
    private lazy var markerLabel = setupMarkerLabel()
    
    
    deinit {
        print("-->startimes: TLDGLineChartController func: deinit")
    }
}

// MARK: - 生命周期
extension TLDGLineChartController {
    
    override func viewDidLoad() {
        super.viewDidLoad()
        view.backgroundColor = .white
        
        setupMakeAddSubviews()
        setupMakeLayoutSubviews()
        setupMakeInitialize()
        
    }
}

// MARK: - Private
private extension TLDGLineChartController {
    
    /// 折线
    func setupChartView() -> LineChartView {
        let view = LineChartView()
        view.backgroundColor = .red
        
        // 无数据配置
        view.noDataText = "暂无数据"
        view.noDataFont = UIFont.systemFont(ofSize: 12)
        view.noDataTextColor = .green
        view.noDataTextAlignment = .center
        
        // 手势配置
        // 取消Y轴缩放
        view.scaleYEnabled = false
        view.scaleXEnabled = true
        // 触控放大
        view.pinchZoomEnabled = true
        // 取消双击缩放
        view.doubleTapToZoomEnabled = true
        // 是否支持拖拽
        view.dragEnabled = true
        // 拖拽后是否有惯性效果
        view.dragDecelerationEnabled = true
        // 拖拽后惯性效果的摩擦系数(0~1)，数值越小，惯性越不明显
        view.dragDecelerationFrictionCoef = 0.9
        
        // 描述及图例样式
        view.chartDescription.enabled = true
        view.chartDescription.text = "Toj-Test"
        view.chartDescription.font = UIFont.systemFont(ofSize: 12)
        view.chartDescription.textColor = .blue
        view.chartDescription.textAlign = .center
        
        // 动画
        view.animate(xAxisDuration: 1)
        
        // 设置能够显示的数据数量
        view.maxVisibleCount = 999
        
        // 绘制左边轴, 默认true, 不需要设置
        view.leftAxis.enabled = true
        // 不绘制右边轴
        view.rightAxis.enabled = false
        
        view.delegate = self
        
        return view
    }
    
    func setupMarkerLabel() -> UILabel {
        let view = UILabel()
        view.frame = CGRectMake(0, 0, 35, 25)
        view.font = UIFont.systemFont(ofSize: 15)
//        view.textAlignment = .natural
        view.text = "12345"
        view.textColor = .white
        view.backgroundColor = .gray
        return view
    }
    
    func setupLineChartDataSet(y datas: [ChartDataEntry], name: String) -> LineChartDataSet {
        let set = LineChartDataSet(entries: datas, label: name)
        //设置折线的样式
        //折线宽度
        set.lineWidth = 2
        set.circleRadius = 3
        set.setCircleColor(.green)
        
        //是否在拐点处显示数据
        set.drawValuesEnabled = false
        set.valueFormatter = DataValueFormatter()
        //折线拐点处显示数据的颜色
        set.valueColors = [UIColor.brown]
        set.highlightColor = .gray
        // 折线拐点样式
        // 是否绘制拐点
        set.drawCirclesEnabled = false
        // 是否填充颜色
        set.drawFilledEnabled = false
        
        //是否开启绘制阶梯样式的折线图
        // yset.drawSteppedEnabled = false
        return set
    }
    
    /**
     折线图的数据data: LineChartData
     外层需要LineChartDataSet包装, 每一个 LineChartData 就是一条线
     */
    func setChartData() {
        // X轴的数据
        var xvals = [String]()
        for i in 0..<50 {
            if i < 30 {
                xvals.append("02-\(i)")
                continue
            }
            xvals.append("03-\(i)")
        }
        
        var yval = Double.zero
        var xval = Double.zero
        
        // Y轴的数据 两条折线图, 需要两个数组数据
        //对应Y轴上面需要显示的数据
        var y1Vals = [ChartDataEntry]()
        
        for i in 0..<50 {
            xval = Double(i)
            yval = Double(arc4random_uniform(100))
            let entry = ChartDataEntry(x: xval, y: yval)
            y1Vals.append(entry)
        }
        
        var y2Vals = [ChartDataEntry]()
        for i in 0..<50 {
            xval = Double(i)
            yval = Double(arc4random_uniform(100))
            let entry = ChartDataEntry(x: xval, y: yval)
            y2Vals.append(entry)
        }
        
        let yset = setupLineChartDataSet(y: y1Vals, name: "y1")
        // 折线颜色
        yset.setColor(.green)
        //添加第二个LineChartDataSet对象
        let yset2 = setupLineChartDataSet(y: y2Vals, name: "y2")
        yset2.setColor(.blue)
        
        // 创建 LineChartData 对象, 此对象就是lineChartView需要最终数据对象
        let data = LineChartData(dataSets: [yset, yset2])
        //文字字体
        data.setValueFont(UIFont.systemFont(ofSize: 11))
        //文字颜色
        data.setValueTextColor(.brown)
        
        lineChartView.data = data
    }
}

// MARK: - 代理事件
extension TLDGLineChartController: ChartViewDelegate {
    
    /**
     当一个值在图表中被选中时调用
     
     - parameter chartView: 用户名
     - parameter entry: 选中的表项
     - parameter highlight: 包含突出显示位置信息的相应突出显示对象, 如dataSetIndex等
     */
    func chartValueSelected(_ chartView: ChartViewBase, entry: ChartDataEntry, highlight: Highlight) {
        print("-->startimes: chartValueSelected: \(entry.y)%")
        
        //改变选中的数据时候，label的值跟着变化
        //将点击的数据滑动到中间
        markerLabel.text = "\(entry.y)%"
        
        guard let dependenc = lineChartView.data?.dataSet(at: highlight.dataSetIndex)?.axisDependency else { return }
        
        lineChartView.centerViewToAnimated(
            xValue: entry.x,
            yValue: entry.y,
            axis: dependenc,
            duration: 1
        )
    }
    
    /// 当用户停止在图表上的值之间平移时调用
    func chartViewDidEndPanning(_ chartView: ChartViewBase) {
        print("-->startimes: chartViewDidEndPanning")
    }
    
    // 当没有选择任何内容或“取消选择”时调用
    func chartValueNothingSelected(_ chartView: ChartViewBase) {
        print("-->startimes: chartValueNothingSelected")
    }
    
    // 通过缩放手势缩放图表时的回调函数。
    func chartScaled(_ chartView: ChartViewBase, scaleX: CGFloat, scaleY: CGFloat) {
        guard let lineChartView = chartView as? LineChartView else { return }
        let scaleX = lineChartView.scaleX
        print("-->startimes: chartScaled x: \(scaleX)")
    }
    
    // 通过拖动手势移动/翻译图表时的回调函数
    func chartTranslated(_ chartView: ChartViewBase, dX: CGFloat, dY: CGFloat) {
        
        print("-->startimes: chartTranslated x: \(dX), y:\(dY)")
    }
    
    // 当Animator停止动画时回调
    func chartView(_ chartView: ChartViewBase, animatorDidStop animator: Animator) {
        print("-->startimes: animatorDidStop")
    }
}

// MARK: - 初始化设置
private extension TLDGLineChartController {
    
    func setupMakeAddSubviews() {
        view.addSubview(lineChartView)
    }
    
    func setupMakeLayoutSubviews() {
        lineChartView.frame = view.bounds
    }
    
    func setupMakeInitialize() {
        setupXAxis()
        setupLeftYAxis()
        setupRightYAxis()
        setupMarker()
        
        setChartData()
    }
    
    func setupXAxis() {
        let xAxis = lineChartView.xAxis
        //设置重复的值不显示
        xAxis.granularityEnabled = true
        //设置x轴数据在底部
        xAxis.labelPosition = .bottom
        xAxis.gridColor = .clear
        //文字颜色
        xAxis.labelTextColor = .black
        xAxis.axisLineColor = .gray
        xAxis.valueFormatter = DataValueFormatter()
    }
    
    func setupLeftYAxis() {
        //获取左边Y轴
        let laxis = lineChartView.leftAxis
        // Y轴label数量, 数值不一定,
        // 如果forceLabelsEnabled等于YES, 则强制绘制制定数量的label, 但是可能不平均
        laxis.labelCount = 5
        //不强制绘制指定数量的label
        laxis.forceLabelsEnabled = false
        
        //设置Y轴的最小值
        // leftAxis.axisMinValue = 0
        //设置Y轴的最大值
        // leftAxis.axisMaxValue = 105
        //是否将Y轴进行上下翻转
        laxis.inverted = false
        //Y轴颜色
        laxis.axisLineColor = .clear
        //label位置
        laxis.labelPosition = .outsideChart
        //文字颜色
        laxis.labelTextColor = .black
        //文字字体
        laxis.labelFont = UIFont.systemFont(ofSize: 10)
        //网格线颜色
        laxis.gridColor = .gray
        //开启抗锯齿
        laxis.gridAntialiasEnabled = false
        //设置y轴的数据格式
        laxis.valueFormatter = DataValueFormatter()
    }
    
    func setupRightYAxis() {
        //获取左边Y轴
        let raxis = lineChartView.rightAxis
        // Y轴label数量, 数值不一定,
        // 如果forceLabelsEnabled等于YES, 则强制绘制制定数量的label, 但是可能不平均
        raxis.labelCount = 5
        //不强制绘制指定数量的label
        raxis.forceLabelsEnabled = false
        
        //设置Y轴的最小值
        // raxis.axisMinValue = 0
        //设置Y轴的最大值
        // raxis.axisMaxValue = 105
        //是否将Y轴进行上下翻转
        raxis.inverted = false
        //Y轴颜色
        raxis.axisLineColor = .clear
        //label位置
        raxis.labelPosition = .outsideChart
        //文字颜色
        raxis.labelTextColor = .black
        //文字字体
        raxis.labelFont = UIFont.systemFont(ofSize: 10)
        //网格线颜色
        raxis.gridColor = .gray
        //开启抗锯齿
        raxis.gridAntialiasEnabled = false
        //设置y轴的数据格式
        raxis.valueFormatter = DataValueFormatter()
    }
    
    func setupMarker() {
        let marker = MarkerView()
        marker.offset = CGPointMake(-999, -8)
        marker.chartView = lineChartView
        marker.addSubview(markerLabel)
        
        lineChartView.marker = marker
    }
}
